
# q-expansion of newform 471.2.a.e, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 471
weight = 2
poly_data = [-15, 173, -290, -349, 711, 294, -500, -106, 149, 17, -20, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-3, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, -5, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-9, 7, 25, -2, -10, 0, 1, 0, 0, 0, 0, 0], 1], [[9, 74, 2, -293, -156, 250, 116, -94, -27, 16, 2, -1], 2], [[20, -175, -21, 549, 100, -422, -64, 134, 14, -19, -1, 1], 2], [[34, -219, -19, 615, 98, -444, -64, 136, 14, -19, -1, 1], 2], [[-25, 285, -100, -776, 64, 553, -14, -162, 1, 21, 0, -1], 1], [[-61, 502, 56, -1475, -356, 1118, 244, -364, -55, 54, 4, -3], 2], [[-38, 343, -80, -911, 31, 628, -3, -177, 0, 22, 0, -1], 1], [[-49, 486, -86, -1359, -34, 986, 50, -297, -13, 40, 1, -2], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0], [0, 0, -1, 1, -1, -1, 1, 1, -1, 1, -1, 1], [1, 0, 1, -1, 0, 0, -1, 0, 0, -1, 0, 0], [1, 0, 0, 0, 0, 1, -1, 0, 0, 0, 0, -1], [1, 1, -1, 1, 0, -1, 1, 1, -1, 1, -1, 1], [0, 0, 0, 0, 0, 1, 0, -1, 0, -1, 2, 0], [1, 0, -1, 1, 0, 0, 0, 0, 0, 0, 2, -1], [1, 1, 0, -2, 1, 0, 0, 0, 0, 0, 0, 0], [0, 1, 1, -1, 1, 0, -1, 0, 2, 1, 0, 0], [0, 2, 2, -2, 2, 2, 0, 0, 1, -2, 0, 0], [3, 0, -2, 2, -1, 0, 1, 0, -1, 0, 1, 0], [0, 0, 1, -1, 0, 1, 0, -2, 0, -1, 0, -1], [-2, 0, 0, 0, -1, -2, -1, 1, 0, 0, -1, 0], [0, 0, 2, 0, -1, 1, -1, 0, -1, 1, -1, 0], [2, 0, -2, 2, -1, -2, 1, 1, 1, 2, 1, 0], [2, 0, -1, 1, -1, -3, 1, 1, -1, 1, -1, -1], [0, 2, -1, 1, 1, -1, 1, 1, -1, 1, -1, -1], [2, 0, -3, 1, -1, -3, 1, 1, 1, 3, -1, -1], [0, 0, 0, -2, 1, 0, -1, 1, -1, -2, -1, 0], [-1, 0, 1, -1, 1, 0, 1, 1, -1, 0, 1, 1], [-1, -2, 1, 1, 0, 0, 1, 2, -2, 1, 0, 2], [0, -2, 0, 0, 1, 2, -1, -1, -1, 0, 1, 0], [1, 0, 3, -3, -1, 2, 0, -1, -1, -1, -1, 0], [0, -6, -1, 1, -1, -1, -1, -1, -1, -1, 1, -1], [-5, -2, 5, -3, 1, 3, -3, -2, 0, -3, -1, 1], [-2, -1, 0, 2, 1, -1, 1, 0, 0, 2, -2, 1], [1, 2, -4, 2, 0, 1, 1, 0, -2, 0, 0, -1], [1, -1, 2, 0, 2, 2, 1, -1, 1, -2, 1, 0], [-1, 2, 0, 0, 0, 0, 0, 1, -1, -2, 0, 0], [-3, -2, -2, 2, 0, -1, 0, 0, 0, 3, 0, 0], [2, 0, 0, 2, 0, -2, 2, 2, 0, 2, 0, 0], [0, 2, -1, -1, 0, -1, 2, 2, 3, 1, 0, 1], [1, 2, 5, -3, 1, 3, -3, 0, 1, 1, -1, -1], [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [2, -2, 1, -1, 3, 5, -1, -1, -1, -3, 3, -1], [-4, 2, 3, -1, 0, -1, 0, 0, 2, -1, 0, 1], [2, 0, -3, 1, 1, -1, 1, 1, -1, -1, -1, 1], [-2, -2, 2, -4, 0, -1, 0, 1, -2, -1, -2, 0], [0, 0, -3, 1, -2, -5, 0, 0, 0, 3, -2, -3], [-2, 4, -2, 0, 3, 2, 3, 1, 0, -2, 1, 2], [4, -2, 6, -2, 1, 4, -3, -3, 1, -2, 3, 0], [1, -2, -2, 2, 0, 1, -1, 0, -2, -2, 2, -3], [0, -4, -2, 4, -5, -4, 1, 3, -3, 2, -3, 2], [3, 0, 2, -2, -2, 2, 0, -1, 2, 0, 2, 0], [-2, 0, -2, 4, -1, -2, 1, -1, 1, 0, 3, 0], [-3, 2, -1, -1, 0, -1, 0, -1, 3, -1, 2, -1], [-2, -4, -3, 1, 1, 1, 1, -1, 1, 1, 1, 1], [5, 0, 1, -1, 1, 0, -2, -1, -1, 1, -3, 0], [-4, -2, -4, 2, -3, -4, 1, 3, -1, 4, -3, 2], [8, 2, -3, 1, 2, 1, 2, 0, -2, -3, 2, 3], [-3, -2, -5, 7, -3, -5, 3, 2, 2, 3, -1, 1], [2, -2, 0, 2, -2, -2, 0, 0, -2, 2, -2, 0], [-2, 4, -4, 0, 1, 2, -1, -1, 1, -2, 1, -2], [7, -4, 1, -3, 1, 0, -1, 1, -3, 0, 1, 1], [-3, 4, -1, -1, 0, -3, 2, 1, 0, 1, -2, 1], [2, -3, 7, -3, -2, 2, -2, 1, 1, -1, -1, 0], [3, -4, 5, -1, 2, 4, -3, -6, 2, -3, 2, 0], [1, -1, -3, 3, -1, -3, 4, 2, 2, 3, 2, 1], [-3, 4, 0, 2, 0, -1, 0, 2, 2, 5, 0, 0], [-3, 0, 6, -2, 3, 4, -5, -4, 5, -6, 3, -4], [0, 0, -6, 2, -1, -2, 3, 3, -1, 4, -1, 0], [8, -3, 0, -2, 2, 5, -4, -3, -1, -2, 1, -1], [-1, -2, 4, 0, -3, 3, -2, 1, -3, 0, -1, 1], [0, 6, -1, 1, 2, 1, 0, 2, 4, 1, 0, -1], [0, 4, -6, 6, -1, -4, 3, 1, -1, 2, 1, -2], [-6, 2, -3, 1, 1, -1, 3, -1, -3, -1, 1, 1], [5, 0, -2, 0, 0, 3, -1, -2, 0, -2, 2, -1], [8, 0, 3, -3, 0, -1, -4, -2, -2, 1, -4, -1], [-6, -4, 7, -7, 1, 4, -5, -2, -1, -4, -1, -1], [-1, 2, -6, 6, 0, 0, 0, 1, -6, 2, 0, -2], [-4, -2, -1, 3, 1, -1, -1, 1, 1, -1, -1, -1], [-5, 2, 3, -1, -1, -1, 5, 2, 1, 3, -3, 7], [-7, 4, 0, 0, -2, 0, 0, -1, -1, -2, 0, 2], [-1, 0, 5, -3, 1, 4, -6, 1, 5, 1, -1, -4], [0, -6, 1, -3, 1, -1, 1, 3, -3, 1, -5, -1], [0, 2, -2, -2, -1, -4, 1, 1, -4, -2, -3, 4], [4, 4, -7, 5, 0, -7, 4, 4, -2, 3, -4, 3], [-4, 4, 10, -8, 5, 10, -5, -3, 1, -6, -1, -2], [-4, 6, -5, 3, -1, -5, 5, 3, 1, 3, 1, 3], [0, 4, -8, 4, -3, -2, 5, 1, -3, 4, 1, 0], [0, 2, 1, 1, 1, 1, 3, -1, -1, 1, 1, -1], [1, 0, -2, 2, 2, -2, 4, 5, -4, 0, 2, 0], [-10, 6, 0, -2, 3, 0, 1, -1, 4, 0, 3, 2], [4, -8, -4, 0, 2, 2, 0, 0, -1, -4, 2, 2], [-1, 4, -1, 1, 2, 0, -3, 2, 4, 5, -2, 0], [5, 4, -3, 3, -2, -2, -1, -2, 2, 5, -2, -2], [-5, 2, 1, -3, 1, 5, -1, -4, 3, -3, 3, 1], [-14, -6, 0, 2, -2, -2, 0, -2, -4, -2, -2, 4], [-2, 2, 3, -1, 1, 0, 1, 2, 9, 0, -1, -1], [-8, -2, 10, -2, -1, 0, -3, -3, -3, -2, -3, 4], [-6, 0, 0, 0, 4, 5, -6, -3, 0, -3, 2, -6], [4, 4, -2, 4, -1, -2, 1, -1, -1, 0, 5, 2], [-7, 4, -10, 2, -1, -6, 3, 6, -2, 4, -3, 2], [3, 2, -8, 2, 1, -3, 3, 3, 1, 1, 3, -6], [13, 4, -2, 0, 2, -3, 0, 0, 0, 1, 2, 0], [7, -3, 11, -7, 1, 9, -6, -6, 4, -3, 0, -1], [-6, 2, 0, 0, -1, 0, 1, -1, 5, 2, 1, 2], [7, 3, 8, -2, 2, 6, -5, -3, 7, 0, -1, -2], [2, 2, 2, -2, 1, 8, -1, -7, 5, -2, 3, -2], [-9, 0, 2, 2, -3, -2, -3, -2, 0, 4, -1, -2], [-2, -4, 5, -3, 0, 1, -4, 2, 3, 3, -2, 5], [2, -2, 8, -4, 1, 4, 1, -5, 3, -4, 3, 0], [-4, 9, 0, -2, 6, 3, 2, 3, 3, -2, 1, 1], [-2, -2, 0, -2, 4, 3, 4, -1, -2, -5, 2, -2], [5, -6, 8, -2, -5, 3, -2, -5, -3, -6, 3, 1], [-6, 6, -2, -2, 3, 1, -1, 2, -3, -5, 3, -2], [11, -2, 0, -2, -1, 1, -4, 3, -1, 0, -1, -3], [-5, 0, -2, 2, -3, -6, 7, 6, -3, 2, -3, 6], [-8, -4, -2, 2, 1, -4, -1, 1, 3, 0, 1, -2], [-1, -6, 7, -1, -6, -2, 1, 2, 0, 1, -2, 0], [7, -3, -10, 8, -5, -4, 4, 0, 0, -2, 4, 2], [-4, 2, -2, -2, 3, -4, 1, 1, 5, -2, 1, 0], [4, 0, 2, -10, 5, 4, -3, -1, 3, -6, -3, 2], [-7, 6, -2, 2, 0, -6, 0, -1, 0, 0, -2, 4], [0, 4, 4, -2, 5, 6, -1, -3, 5, -2, 3, 2], [2, 4, -4, 2, -5, -8, -1, -3, -1, -2, 1, 0], [-4, -8, 1, -3, 0, -1, -4, -2, 0, 1, -2, 1], [-8, -3, -1, 5, -6, -2, 2, -1, 1, -1, -1, 4], [6, -8, -2, 2, -1, -6, 5, 1, -3, 2, -3, 0], [4, -4, 2, -2, 1, -2, -3, -1, 5, 4, 1, 0], [-8, 4, -2, -4, 5, 1, -3, 2, 5, 1, -3, -4], [-2, -6, 6, -4, 0, -4, -4, 0, 0, 0, -4, 0], [1, 0, -1, 3, 3, 2, 5, -3, 3, -2, 3, 3], [6, -4, -4, 4, -4, 4, -6, -4, -2, 0, 0, -8], [14, -4, -7, 11, -2, -5, 2, 4, -5, 7, -2, 1], [2, 4, -7, 5, -4, -5, 6, 4, -2, 11, -2, 3], [5, -1, -4, 4, -3, 2, 2, 0, -4, 6, 4, 0], [13, -1, 0, -6, 2, 0, -3, 1, -1, 0, -5, 0], [-6, -4, 3, 5, -3, -5, 3, 5, 1, 1, -3, 1], [-1, -6, 2, -6, 4, 10, 0, -7, -2, -8, 6, -4], [2, 6, 0, -4, -3, -2, -5, -1, 1, 0, -1, -2], [25, -4, -1, -1, -5, -2, -3, -3, 3, 2, -1, -1], [-8, 1, 3, -7, 4, 4, -2, 1, -3, -5, -1, 0], [-11, -4, 10, -8, 6, 9, -7, -4, 2, -8, 4, -3], [9, -8, -9, 3, -4, 1, 2, 1, -4, 5, -2, 1], [10, 0, 7, -1, -2, 1, 2, 0, -2, -1, 4, 3], [10, 2, -3, -3, 4, 3, 0, -2, -3, -1, 0, 1], [5, 0, -4, 0, -3, -4, -3, 4, -1, 2, -3, -2], [9, -10, -4, 4, -3, 1, -2, -1, 3, 2, 3, -5], [-5, -2, 7, 3, -4, 1, 2, 5, -6, 3, -6, 5], [0, -12, -2, 6, -5, -8, 5, 1, -7, 4, -5, 2], [6, -15, 6, 0, 0, 5, -4, -3, -5, -4, 1, -3], [-1, -4, 15, -13, 2, 8, -6, -4, 0, -10, 0, 1], [-5, 3, -6, 2, 5, 0, 2, 0, 2, 0, 6, 0], [12, -2, -5, 7, -9, -11, 5, 5, -2, 9, -5, 5], [1, -6, 4, 0, -3, 2, -1, 0, -7, 0, -3, 4], [-5, 0, 4, 0, -2, -2, -2, 3, 5, 6, -4, -4], [-4, 8, 13, -15, 6, 17, -8, -8, 2, -11, 4, -5], [1, 6, 14, -6, 4, 9, -2, -2, 0, -5, 0, 2], [3, 4, 2, 2, -4, -4, 0, 1, 0, 4, -2, 0], [12, 0, 5, -5, 7, 9, -5, -5, 6, -5, 7, -3], [6, 8, -7, 7, -3, -9, 3, 3, 1, 7, -3, 5], [2, -10, -4, 8, -4, -4, 2, 2, -8, 0, -2, 0], [8, 2, 1, 3, -2, -3, 2, 0, 2, 7, -6, 5], [13, 2, 11, -5, 4, 4, -3, -6, 8, -3, 0, 4], [6, 2, -8, 8, -1, -4, -1, 1, -1, 0, 7, -6], [8, 6, -8, 2, -1, -14, 5, 7, 3, 4, -5, 6], [-13, 2, 1, -7, 8, 7, -2, -1, 1, -7, 2, -1], [7, 12, -6, 2, -2, -7, 1, 2, -4, 4, -2, 1], [6, 4, -7, -1, -5, -5, 5, 1, -1, -1, 3, 1], [9, 4, 8, -10, 1, 7, -11, -5, 1, -11, 3, -4], [-5, 2, 4, -2, -5, -3, 0, 3, 3, 6, -5, 7], [-7, -4, 11, -5, 0, 5, 4, -1, -1, -5, 0, 3], [2, 0, 1, 1, 2, 3, -4, 2, 8, 3, 4, -5], [-16, 12, -6, -2, -4, -8, 4, 2, -4, 0, -6, 4]]
